
/* Include libraries. */
#include "mex.h"
#include "math.h"

/* Specify the number of outputs here. */
#define NY 4

/* State equations. */
void compute_dx(
    double *dx,  /* Vector of state derivatives (length nx). */
    double t,    /* Time t (scalar). */
    double *x,   /* State vector (length nx). */
    double *u,   /* Input vector (length nu). */
    double **p,  /* p[j] points to the j-th estimated model parameters (a double array). */
    const mxArray *auxvar  /* Cell array of additional data. */
   )
{
    double K_12, K_1h, K_1g, K_1r, K_1u, K_23, K_2h, K_2r, K_2u, K_3h, K_3r, K_3u, K_hg, K_hr, K_hu, K_1l, K_2l, K_3l, K_hl, K_gv, K_rv, k1_s, k2_s, k3_s, kh_s, k1_d, k2_d, k3_d, kh_d, kg_d, kr_d, m1, m2, m3, mh, mg, mr,K_1p,K_2p,K_3p; /* Estimated model parameters.*/
    
    double c_p = 1012;
	double c_p_c = 800;
	double c_p_s = 460;
    
    double K_11,K_22,K_33,K_hh,K_gg,K_rr, OC_p1,OC_p2;
    
    
	K_12 = p[0][0];
    K_1h = p[1][0];
	K_1g = p[2][0];
	K_1r = p[3][0];
	K_1u = p[4][0];
	K_23 = p[5][0];
	K_2h = p[6][0];
	K_2r = p[7][0];
	K_2u = p[8][0];
	K_3h = p[9][0];
	K_3r = p[10][0];
	K_3u = p[11][0];
	K_hg = p[12][0];
	K_hr = p[13][0];
	K_hu = p[14][0];
	K_1l = p[15][0];
    K_2l = p[16][0];
    K_3l = p[17][0];
    K_hl = p[18][0];
    K_gv = p[19][0];
    K_rv = p[20][0];
	k1_s = p[21][0];
	k2_s = p[22][0];
	k3_s = p[23][0];
	kh_s = p[24][0];
	k1_d = p[25][0];
	k2_d = p[26][0];
	k3_d = p[27][0];
	kh_d = p[28][0];
	kg_d = p[29][0];
	kr_d = p[30][0];
	m1   = p[31][0];
	m2   = p[32][0];
	m3   = p[33][0];
	mh   = p[34][0];
	mg   = p[35][0];
	mr   = p[36][0];
    K_1p   = p[37][0];
    K_2p   = p[38][0];
    K_3p   = p[39][0];
    
    
    
    K_23 = K_12*K_23;
    K_2h = K_1h*K_2h;
    K_3h = K_1h*K_3h;
    K_2r = K_1r*K_2r;
    K_3r = K_1r*K_3r;
    K_hg = K_1g*K_hg;
    k2_s = k1_s*k2_s;
    k3_s = k1_s*k3_s;
    
    OC_p1 = u[8];
    OC_p2 = u[9];
    K_1l = K_1l *OC_p1;
    K_2l = K_2l *OC_p1;
    K_3l = K_3l *OC_p2;
    K_hl = K_hl *OC_p2;

	K_11 = K_12+K_1h+K_1g+K_1r      +K_1u+K_1l;
	K_22 = K_12+K_23+K_2h+K_2r      +K_2u+K_2l;
	K_33 = K_23+K_3h+K_3r           +K_3u+K_3l;
	K_hh = K_1h+K_2h+K_3h+K_hg+K_hr +K_3u+K_hl;
	K_gg = K_hg+K_1g                +K_gv;
	K_rr = K_1r+K_2r+K_3r+K_hr      +K_rv;

	dx[0] = -K_11*x[0]+K_12*x[1]          +K_1h*x[3]+K_1g*x[4]+K_1r*x[5] +    K_1l*u[0]                              +K_1u*u[4]+k1_s*u[5]+k1_d*u[6]+K_1p*u[7];
	dx[1] =  K_12*x[0]-K_22*x[1]+K_23*x[2]+K_2h*x[3]          +K_2r*x[5] +    K_2l*u[0]                              +K_2u*u[4]+k2_s*u[5]+k2_d*u[6]+K_2p*u[7];
	dx[2] =            K_23*x[1]-K_33*x[2]+K_3h*x[3]          +K_3r*x[5] +              K_3l*u[1]                    +K_3u*u[4]+k3_s*u[5]+k3_d*u[6]+K_3p*u[7];/*3.etg */
	dx[3] =  K_1h*x[0]+K_2h*x[1]+K_3h*x[2]-K_hh*x[3]+K_hg*x[4]+K_hr*x[5] +              K_hl*u[1]                    +K_hu*u[4]+kh_s*u[5]+kh_d*u[6];/*halll*/
	dx[4] =  K_1g*x[0]                    +K_hg*x[3]-K_gg*x[4]           +                        K_gv*u[2]                              +kg_d*u[6];/*gulv*/
	dx[5] =  K_1r*x[0]+K_2r*x[1]+K_3r*x[2]+K_hr*x[3]          -K_rr*x[5] +                                  K_rv*u[3]                    +kr_d*u[6];/*rad*/

	dx[0]=dx[0]*30/m1/c_p;
	dx[1]=dx[1]*30/m2/c_p;
	dx[2]=dx[2]*30/m3/c_p;
	dx[3]=dx[3]*30/mh/c_p;
	dx[4]=dx[4]*30/mg/c_p_c;
	dx[5]=dx[5]*30/mr/c_p_s;
    

}

/* Output equations. */
void compute_y(
    double *y,   /* Vector of outputs (length NY). */
    double t,    /* Time t (scalar). */
    double *x,   /* State vector (length nx). */
    double *u,   /* Input vector (length nu). */
    double **p,  /* p[j] points to the j-th estimated model parameters (a double array). */
    const mxArray *auxvar  /* Cell array of additional data. */
   )
{


    y[0] = x[0];
    y[1] = x[1];
    y[2] = x[2];
    y[3] = x[3];
}



void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    /* Declaration of input and output arguments. */
    double *x, *u, **p, *dx, *y, *t;
    int     i, np, nu, nx;
    const mxArray *auxvar = NULL; /* Cell array of additional data. */

    if (nrhs < 3) {
        mexErrMsgIdAndTxt("IDNLGREY:ODE_FILE:InvalidSyntax",
        "At least 3 inputs expected (t, u, x).");
    }

    /* Determine if auxiliary variables were passed as last input.  */
    if ((nrhs > 3) && (mxIsCell(prhs[nrhs-1]))) {
        /* Auxiliary variables were passed as input. */
        auxvar = prhs[nrhs-1];
        np = nrhs - 4; /* Number of parameters (could be 0). */
    } else {
        /* Auxiliary variables were not passed. */
        np = nrhs - 3; /* Number of parameters. */
    }

    /* Determine number of inputs and states. */
    nx = mxGetNumberOfElements(prhs[1]); /* Number of states. */
    nu = mxGetNumberOfElements(prhs[2]); /* Number of inputs. */

    /* Obtain double data pointers from mxArrays. */
    t = mxGetPr(prhs[0]);  /* Current time value (scalar). */
    x = mxGetPr(prhs[1]);  /* States at time t. */
    u = mxGetPr(prhs[2]);  /* Inputs at time t. */

    p = mxCalloc(np, sizeof(double*));
    for (i = 0; i < np; i++) {
        p[i] = mxGetPr(prhs[3+i]); /* Parameter arrays. */
    }

    /* Create matrix for the return arguments. */
    plhs[0] = mxCreateDoubleMatrix(nx, 1, mxREAL);
    plhs[1] = mxCreateDoubleMatrix(NY, 1, mxREAL);
    dx      = mxGetPr(plhs[0]); /* State derivative values. */
    y       = mxGetPr(plhs[1]); /* Output values. */


    /* Call function for state derivative update. */
    compute_dx(dx, t[0], x, u, p, auxvar);

    /* Call function for output update. */
    compute_y(y, t[0], x, u, p, auxvar);

    /* Clean up. */
    mxFree(p);
}
